@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Net::Async::Redis::Commands IO::Async::Notifier);
 
-our $VERSION = '1.007';
+our $VERSION = '1.008';
 
 =head1 NAME
 
@@ -270,7 +270,7 @@ sub connect : method {
         );
         return $self->auth($auth) if defined $auth;
         return Future->done;
-    })
+    })->on_fail(sub { delete $self->{connection} });
 }
 
 sub connected { shift->connect }
@@ -349,7 +349,7 @@ sub handle_pubsub_message {
             redis => $self,
             channel => $channel
         );
-        $self->{'pending_' . $k}{$channel}->done($payload);
+        $self->{'pending_' . $k}{$channel}->done($payload) unless $self->{'pending_' . $k}{$channel}->is_done;
     } else {
         $log->warnf('have unknown pubsub message type %s with channel %s payload %s', $type, $channel, $payload);
     }
@@ -396,8 +396,24 @@ sub bus {
 
 sub notify_close {
     my ($self) = @_;
-    $self->configure(on_read => sub { 0 });
-    $_->[1]->fail('Server connection is no longer active', redis => 'disconnected') for @{$self->{pending}};
+    # If we think we have an existing connection, it needs removing:
+    # there's no guarantee that it's in a usable state.
+    if(my $stream = delete $self->{stream}) {
+        $stream->close_now;
+    }
+
+    # Also clear our connection future so that the next request is triggered appropriately
+    delete $self->{connection};
+    # Clear out anything in the pending queue - we normally wouldn't expect anything to
+    # have ready status here, but no sense failing on a failure.
+    !$_->[1]->is_ready && $_->[1]->fail('Server connection is no longer active', redis => 'disconnected') for splice @{$self->{pending}};
+
+    # Subscriptions also need clearing up
+    $_->cancel for values %{$self->{subscription_channel}};
+    $self->{subscription_channel} = {};
+    $_->cancel for values %{$self->{subscription_pattern_channel}};
+    $self->{subscription_pattern_channel} = {};
+
     $self->maybe_invoke_event(disconnect => );
 }
 
@@ -519,9 +535,11 @@ Some other Redis implementations on CPAN:
 
 =head1 AUTHOR
 
-Tom Molesworth <TEAM@cpan.org>
+Tom Molesworth <TEAM@cpan.org>, with patches and input from
+C<< BINARY@cpan.org >>, C<< PEVANS@cpan.org >> and C<< @eyadof >>.
 
 =head1 LICENSE
 
-Copyright Tom Molesworth 2015-2017. Licensed under the same terms as Perl itself.
+Copyright Tom Molesworth and others 2015-2018.
+Licensed under the same terms as Perl itself.
 
